#!/bin/zsh

# ARCHIVE & ERASE
# This script will create a zip of the current FileWave Log files
# Once zipped, the current files will be emptied
# This will occur repeatedly on the set duration
# 2023.11.21 - Author: sean.holden@filewave.com

# NOTE: This script will stop the FileWave Client initially and then restart once done.
# THIS SCRIPT SHOULD THEREFORE NOT BE RAN THROUGH FILEWAVE, BUT DELIVERED ONLY

if [[ "$(whoami)" != "root" ]]
then
	echo "Must be ran as root"
	exit 1
fi

# Coloured output
BOLD='\033[1m'
NC='\033[0m'
RED='\033[1;31m'

function usage {

	echo -e "${BOLD}Usage${NC}

Files included for archive will be not only zipped, but original files will be emptied.

Optional:
  -a [Action the script.  Action a dry run if this option is not specified.  Dry run will echo only]
  -d [Integer specifying the amount of log days to keep (default 10 days)]
  -e [Start from the beginning.  The zip will be erased and the script will run as if first ran]
  -f [Archive Fileset logs from client as well as generic FileWave Client logs.  -g is uneccessary when running this option]
  -g [Archive generic FileWave Client logs (not including Fileset logs).  Fileset logs will remain as is.]
  -r [Rerun the script, but keep zip archive.  Current zip will be presevered and only additional logs included from the above options will be added to the archive if not already zipped]
  -s [Archive FileWave Central logs from computer]
  -x [This options will zip all client logs (same as running -f) and also action the script]"
	log_me "Usage triggered.  Exit 1"
	exit 1
}

# Self log in same location, but with a name that will not be impacted with other logs
log_file="/private/var/log/fwlog_limit.log"

echo "Logging to: $log_file"

# Log store for archive copy
archive_log_directory="/private/var/log/fwclient_log_archives"
archive_log_file="${archive_log_directory}/filewave_client_logs.zip"
archive_log_exclusion="${archive_log_directory}/exclusion_list.txt"

# Header Message for the log file for this script
the_header="---------------HEADER------------------\n$(date)\nArchive logs."

function log_me {

	if [[ "$CMD" != "echo" ]]
	then
		echo "$1" | tee -a "$log_file"
	fi
}

log_me "$the_header"

# Dry run set to default true.  Use optional argument -a to overrule this
dry_run=1

declare -a arguments=()
fileset_logs=0
filewave_central_logs=0
fwclient_logs=0
rerun=0
erase_zip=0

for the_options
do
	case "$the_options" in

		-a)		arguments+=( -a );;
		-d)             arguments+=( -d );;
		-f)             arguments+=( -f );;
		-s)		arguments+=( -s );;
		-g)		arguments+=( -g );;
		-r)		arguments+=( -r );;
		-e)		arguments+=( -e );;
		-x)		arguments+=( -x );;
		-*)             usage;;
		*)              arguments+=( "$the_options" );;
	esac
done

set -- "${arguments[@]}"

while getopts "afsgrexd:" OPTION
do
	: "$OPTION" "$OPTARG"

	case $OPTION in

		a)	dry_run=0;;
		d)      days_to_keep=$OPTARG; if [[ ! $OPTARG =~ ^[0-9]+$ ]]; then usage; fi;;
		f)	fileset_logs=1;;
		s)	filewave_central_logs=1;;
		g)	fwclient_logs=1;;
		r)	rerun=1;;
		e) 	erase_zip=1;;
		x)	dry_run=0;fileset_logs=1;;
	esac
done

# Every x days we will archive and empty current log files
# Integer of days
if [ ! $days_to_keep ]
then
	days_to_keep=10
fi
log_me "Days to keep: $days_to_keep"

case $dry_run in

	1)
		CMD=echo
		LOG_CMD=""
	;;
	0)
		CMD=""
		LOG_CMD=log_me
	;;
	*)
		log_me "Something has gone wrong"
		exit 1
	;;
esac

# Reset the header message for all of the FileWave Client log files
the_header="---------------HEADER------------------\n$(date)"

# Client should be stopped for this process
log_me "Stopping client"

if [ ! -e "$archive_log_directory" ]
then
	$CMD mkdir "$archive_log_directory"
fi

# Do not run until x days old, as defined in the days_to_keep variable
if [ -e "$archive_log_file" ]
then
	if [ $rerun -eq 1 ]
	then
		# Build an exclusion list file
		unzip -l "$archive_log_file" | awk '/[0-9]{2}-[0-9]{2}-[0-9]{4}/ {print "/"substr($0,31)}' > "${archive_log_exclusion}"

#		echo "Exlusion list"
#		cat "${archive_log_exclusion}"
	fi

	# If we have a zip archive, how old is that file?  Only replace every x days
	# When replaced, this script will archive/zip the original log files then empty the original log files

	if [ $erase_zip -eq 1 ]
	then
		$CMD rm "$archive_log_file"
		rerun=0
	else
		$CMD $LOG_CMD "OS type: $os_type"
		last_modified=$(stat -f %m "$archive_log_file")

		date_now=$(date "+%s")
		days_old_secs=$(( date_now - last_modified))

		# Convert to days
		days_old_days=$(( $days_old_secs / 60 / 60 /24 ))

		# Okay, so if the file is less than x days old, nothing to do
		# Otherwise, move the old zip file, ready to create a new zip file.  Old temp zip will be removed at the end.
		if [ $days_old_days -lt $days_to_keep ] && [ $rerun -eq 0 ]
		then
			$CMD $LOG_CMD "File $archive_log_file age in days: $days_old_days"
			$CMD $LOG_CMD "Threshold days before delete: $days_to_keep"
			$CMD $LOG_CMD "Nothing to do"
			exit 0
		else
			if [ ! $CMD ]
			then
				log_me "$(fwcontrol client stop)"
			fi

			if [ $rerun -eq 0 ]
			then
				# Only replace zip file if this is not a rerun
				$CMD $LOG_CMD "Prepare to remove replace old archive"
				$CMD mv "$archive_log_file" "${archive_log_file}.temp"
			fi
		fi
	fi
fi

function reset_log_file {

		$CMD $LOG_CMD "Emptying original log file: $the_file"

		# Rather than actually emptying the original log file, put in a header, note and footer.
		$CMD echo "$the_header" > "${the_file}"
		$CMD echo "Log archived to $archive_log_file" >> "${the_file}"
		$CMD echo "$the_footer" >> "${the_file}"
}

function zip_log {

	the_file="$1"
	the_footer="-------------------FOOTER------------------"

	# Avoid writing to the log of this script and zipping that log at the same time
	if [[ "$the_file" == "$log_file" ]]
	then
		$CMD $LOG_CMD "Will zip this log file after"
		$CMD $LOG_CMD "$the_footer"
		$CMD zip "$archive_log_file" "$the_file"
	else
		if [ ! $CMD ]
		then
			if [ $rerun -eq 0 ]
			then
				if [ -e "${archive_log_exclusion}" ]
				then
					rm "${archive_log_exclusion}"
				fi
			fi

			if [ -e "${archive_log_exclusion}" ]
			then
				zip_response=$(zip "$archive_log_file" "$the_file" -x@"${archive_log_exclusion}" 2>/dev/null)

				if [ $? -ne 0 ]
				then
					log_me "Ignoring file: $the_log"
				else
					log_me "$zip_response"
				fi
			else
				log_me "$(zip $archive_log_file $the_file)"
			fi
		else
			log_me "zip $archive_log_file $the_file"
		fi
	fi

	case $rerun in

		0)
			# Reset the log file if not running a rerun
			reset_log_file
		;;
		1)
#			echo "Resetting log file with rerun: $rerun"
#			echo "File: $the_file"
#			ls -al "${archive_log_exclusion}"
			# On a rerun, only reset log files that are not excluded
			file_excluded=$(grep -m 1 "$the_file" "${archive_log_exclusion}")


			if [ ! "$file_excluded" ]
			then
				reset_log_file
			else
				log_me "File in exclusion list: $file_excluded"
			fi
		;;
	esac
}

# Run a bunch of while loops to find all of the current log files in their different locations
# They can all be added to the same single zip file.

function while_read_user_log {

	if [[ "$2" == "wild" ]]
	then
		while read the_log
		do
			$CMD zip_log "$the_log"
		done< <(find /Users/*"$1"* -type f)
	else
		while read the_log
		do
			$CMD zip_log "$the_log"
		done< <(find /Users/*"$1" -type f)
	fi
}

function while_read_system_log {

	if [[ "$2" == "wild" ]]
	then
		while read the_log
		do
			$CMD zip_log "$the_log"
		done< <(find "$1"* -type f)
	else
		while read the_log
		do
			$CMD zip_log "$the_log"
		done< <(find "$1" -type f)
	fi
}

function remove_client_logs {

	while_read_system_log "/private/var/log/fwcld.log" wild
	while_read_user_log "/Library/Application Support/FileWave/FWGUI.log" wild
}

# User choice logs to be removed
if [ $fileset_logs -eq 1 ]
then
	# Archive Client logs
	remove_client_logs

	# Archive Fileset logs
	while_read_system_log "/private/var/log/fwcld/" wild
	while_read_system_log "/usr/local/etc/FileWaveInstallerLogfiles" wild

elif [ $fwclient_logs -eq 1 ]
then
	# Archive Client logs
	remove_client_logs
fi

# FileWave Central Logs
if [ $filewave_central_logs -eq 1 ]
then
	# Archive FileWave Central logs
	while_read_user_log "/Library/Application Support/FileWave/FileWaveAdmin.log" wild
	while_read_user_log "/Library/Application Support/FileWave/Server Logs"
	while_read_user_log "/Library/Application Support/FileWave/Client Logs/" wild
fi

$CMD zip_log "$log_file"

# Tidy up.  Remove the temp backup zip now we have finished.
if [ -e "${archive_log_file}.temp" ]
then
	$CMD rm "${archive_log_file}.temp"
fi

# Restart the client service
if [ ! $CMD ]
then
	log_me "$(fwcontrol client start)"
fi

exit 0
