#!/bin/zsh

# ARCHIVE & ERASE
# This script will create a zip of the current FileWave Log files
# Once zipped, the current files will be emptied
# This will occur repeatedly on the set duration
# 2023.11.21 - Author: sean.holden@filewave.com

if [[ "$(whoami)" != "root" ]]
then
	echo "Must be ran as root"
	exit 1
fi

# Coloured output
BOLD='\033[1m'
NC='\033[0m'
RED='\033[1;31m'

os_name=$(uname)

if [[ "$os_name" == "Linux" ]]
then
	os_type=$(awk -F "=" '/^ID=/ {print $2}' /etc/os-release | sed 's/"//g')
else
	os_type="macOS"
fi

function usage {

	echo -e "${BOLD}Usage${NC}\nRequired:\n -d [Integer specifying the amount of log days to keep]\n\nOptional:\n -c [Add the script to cron]\n -a [Action the script.  Dry run if this option is not specified.  Dry run will echo only]"
	exit 1 
}

if [ $# -eq 0 ]
then
	usage
fi

# Self log in same location, but with a name that will not be impacted with other logs
log_file="/private/var/log/fwlog_limit.log"

echo "Logging to: $log_file"

# Log store for archive copy
archive_log_directory="/private/var/log/fwxserver_log_archives"
archive_log_file="${archive_log_directory}/filewave_logs.zip"

# Header Message for the log file for this script
the_header="---------------HEADER------------------\n$(date)\nArchive logs."

function log_me {

	if [[ "$CMD" != "echo" ]]
	then
		echo "$1" | tee -a "$log_file"
	fi
}

log_me "$the_header"

# Run once per day at 04:00
cron_job="0 4 * * * ${0:a} "

declare -a arguments=()
# Every x days we will archive and empty current log files
# Integer of days
# days_to_keep=10

# Option to add this script as a cron job
create_cron=""

# Dry run set to default true.  Use optional argument -a to overrule this
dry_run=1

for the_options
do
	case "$the_options" in

		-a)		arguments+=( -a );;
		-d)		arguments+=( -d );;
		-c)		arguments+=( -c );;
		-*)		usage;;
		*)		arguments+=( "$the_options" );;
	esac
done

set -- "${arguments[@]}"

while getopts "acd:" OPTION
do
	: "$OPTION" "$OPTARG"
	[ $OPTARG ] || if [[ "$OPTION" == "d" ]]; then usage; fi

	case $OPTION in

		a)	dry_run=0;;
		d)      days_to_keep=$OPTARG; if [[ ! $OPTARG =~ ^[0-9]+$ ]]; then usage; fi;;
		c)      create_cron=1;;
	esac
done

log_me "Days to keep: $days_to_keep"

if [ $create_cron ]
then
	echo "$cron_job -d ${days_to_keep} -a" > ~/cron_job.txt
	/usr/bin/crontab ~/cron_job.txt
	rm ~/cron_job.txt

	exit 0
fi

case $dry_run in

	1)
		CMD=echo
		LOG_CMD=""
	;;
	0)
		CMD=""
		LOG_CMD=log_me
	;;
	*)
		echo something has gone wrong
		exit 1
	;;
esac

# Reset the header message for all of the FileWave Server log files
the_header="---------------HEADER------------------\n$(date)"

# Server should be stopped for this process
log_me "Stopping server" 

if [ ! -e "$archive_log_directory" ]
then
	$CMD mkdir "$archive_log_directory"
fi

# Do not run until x days old, as defined in the days_to_keep variable
if [ -e "$archive_log_file" ]
then
	# If we have a zip archive, how old is that file?  Only replace every x days
	# When replaced, this script will archive/zip the original log files then empty the original log files

        case $os_type in 

		"macOS")
			$CMD $LOG_CMD "OS type: $os_type"
			last_modified=$(stat -f %m "$archive_log_file")
		;;
                "debian"|"centos")
			$CMD $LOG_CMD "OS type: $os_type"
			last_modified=$(stat -c %Y "$archive_log_file")
		;;
		*)
			log_me "Server OS $os_type is not configured in this script"
			exit 1
		;;
	esac

	date_now=$(date "+%s")
	days_old_secs=$(( date_now - last_modified))

	# Convert to days
	days_old_days=$(( $days_old_secs / 60 / 60 /24 ))

	# Okay, so if the file is less than x days old, nothing to do
	# Otherwise, move the old zip file, ready to create a new zip file.  Old temp zip will be removed at the end.
	if [ $days_old_days -lt $days_to_keep ]
	then
		$CMD $LOG_CMD "File $archive_log_directory age in days: $days_old_days"
		$CMD $LOG_CMD "Threshold days before delete: $days_to_keep"
		$CMD $LOG_CMD "Nothing to do"
		exit 0
	else
		if [ ! $CMD ]
		then
			log_me "$(fwcontrol server stop)"
		fi

		$CMD $LOG_CMD "Prepare to remove replace old archive"
		$CMD mv "$archive_log_file" "${archive_log_file}.temp"
	fi
fi

function zip_log {

	the_file="$1"
	the_footer="-------------------FOOTER------------------"

	# Avoid writing to the log of this script and zipping that log at the same time
	if [[ "$the_file" == "$log_file" ]]
	then
		$LOG_CMD "Will zip this log file after"
		$LOG_CMD "$the_footer"
		$CMD zip "$archive_log_file" "$the_file"
	else
		if [ ! $CMD ]
		then
			log_me "$(zip $archive_log_file $the_file)"
		else
			$CMD "zip $archive_log_file $the_file"
		fi
	fi

	$CMD $LOG_CMD "Emptying original log file: $the_file"

	# Rather than actually emptying the original log file, put in a header, note and footer.
	$CMD echo "$the_header" > "${the_file}"
	$CMD echo "Log archived to $archive_log_file" >> "${the_file}"
	$CMD echo "$the_footer" >> "${the_file}"	
}

# Run a bunch of while loops to find all of the current log files in their different locations
# They can all be added to the same single zip file.
while read usr_log_file
do
	$CMD zip_log "$usr_log_file"
done< <(find /usr/local/filewave -iname "*.log" -type f)

while read usr_other_log_file
do
	$CMD zip_log "$usr_other_log_file"
done< <(find /usr/local/filewave -iname "*.log*[0-9]" -type f)

while read private_log_file
do
	$CMD zip_log "$private_log_file"
done< <(find /private/var/log -name "fwx*" -type f)

while read private_other_log_file
do
        $CMD zip_log "$private_other_log_file" 
done< <(find /private/var/log -iname "fwx*.log*[0-9]" -type f)

$CMD zip_log "$log_file"

# Tidy up.  Remove the temp backup zip now we have finished.
if [ -e "${archive_log_file}.temp" ]
then
	$CMD rm "${archive_log_file}.temp"
fi

# Restart the server service
if [ ! $CMD ]
then
	log_me "$(fwcontrol server start)"
fi

exit 0
